"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var api_1 = require("@src/api");
var defaultOpts = {
    enableHighAccuracy: false,
    timeout: Infinity,
    maximumAge: 0
};
function useGeolocation(options, runOnMount) {
    if (options === void 0) { options = {}; }
    if (runOnMount === void 0) { runOnMount = true; }
    options = Object.assign({}, defaultOpts, options);
    // Note: surprisingly the watchId can be 0 (not positive) so
    // we have to check if watchId !== null every time
    var watchId = null;
    var geoInitData = {
        loading: false,
        accuracy: null,
        altitude: null,
        altitudeAccuracy: null,
        heading: null,
        latitude: null,
        longitude: null,
        speed: null,
        timestamp: null
    };
    var isTracking = api_1.ref(false);
    var geo = api_1.ref(__assign({}, geoInitData));
    var onEventError = function (error) {
        if (watchId === null)
            return;
        geo.value.loading = false;
        geo.value.error = {
            code: error.code,
            message: error.message
        };
        isTracking.value = false;
    };
    var handleGeolocation = function (_a) {
        var coords = _a.coords, timestamp = _a.timestamp;
        geo.value = {
            loading: false,
            accuracy: coords.accuracy,
            altitude: coords.altitude,
            altitudeAccuracy: coords.altitudeAccuracy,
            heading: coords.heading,
            latitude: coords.latitude,
            longitude: coords.longitude,
            speed: coords.speed,
            timestamp: timestamp
        };
        isTracking.value = true;
    };
    var start = function () {
        if (watchId !== null)
            return;
        geo.value.loading = true;
        geo.value.timestamp = Date.now();
        navigator.geolocation.getCurrentPosition(handleGeolocation, onEventError, options);
        watchId = navigator.geolocation.watchPosition(handleGeolocation, onEventError, options);
    };
    var stop = function () {
        if (watchId === null)
            return;
        navigator.geolocation.clearWatch(watchId);
        watchId = null;
        isTracking.value = false;
    };
    api_1.onMounted(function () { return runOnMount && start(); });
    api_1.onUnmounted(stop);
    return { isTracking: isTracking, geo: geo, start: start, stop: stop };
}
exports.useGeolocation = useGeolocation;
