"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var throttle_debounce_1 = require("throttle-debounce");
var api_1 = require("@src/api");
exports.idleEventsList = [
    'mousemove',
    'mousedown',
    'resize',
    'keydown',
    'touchstart',
    'wheel'
];
var oneMinute = 60e3;
function useIdle(ms, events, runOnMount) {
    if (ms === void 0) { ms = oneMinute; }
    if (events === void 0) { events = exports.idleEventsList; }
    if (runOnMount === void 0) { runOnMount = true; }
    var timeout = null;
    var isIdle = api_1.ref(false);
    var isTracking = api_1.ref(false);
    var handleChange = throttle_debounce_1.throttle(50, function () {
        isIdle.value = false;
        if (timeout)
            clearTimeout(timeout);
        timeout = setTimeout(function () {
            isIdle.value = true;
        }, ms);
    });
    var handleVisibility = function () {
        if (!document.hidden)
            return;
        handleChange();
    };
    var start = function () {
        if (isTracking.value)
            return;
        events.forEach(function (evtName) { return document.addEventListener(evtName, handleChange); });
        document.addEventListener('visibilitychange', handleVisibility);
        // Initialize it since the events above may not run immediately
        handleChange();
        isTracking.value = true;
    };
    var stop = function () {
        if (!isTracking.value)
            return;
        events.forEach(function (evtName) {
            return document.removeEventListener(evtName, handleChange);
        });
        document.removeEventListener('visibilitychange', handleVisibility);
        // Stop timer if it is still running
        if (timeout)
            clearTimeout(timeout);
        // Restore initial status
        timeout = null;
        isIdle.value = false;
        isTracking.value = false;
    };
    api_1.onMounted(function () { return runOnMount && start(); });
    api_1.onUnmounted(stop);
    return { isIdle: isIdle, isTracking: isTracking, start: start, stop: stop };
}
exports.useIdle = useIdle;
