"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var api_1 = require("@src/api");
var errorMsg = 'IntersectionObserver is not supported, please install a polyfill';
function useIntersection(elRef, options, runOnMount) {
    if (options === void 0) { options = {}; }
    if (runOnMount === void 0) { runOnMount = true; }
    var observedEntry = api_1.ref(null);
    var handleObserver = function (entries) {
        observedEntry.value = entries[0];
    };
    var observer = null;
    var start = function () {
        if (!('IntersectionObserver' in window))
            throw new Error(errorMsg);
        // Do not start if the observer is already initialized
        // or the elRef does not exist
        if (observer || !elRef.value)
            return;
        observer = new IntersectionObserver(handleObserver, options);
        observer.observe(elRef.value);
    };
    var stop = function () {
        if (!observer)
            return;
        observer.disconnect();
        observer = null;
    };
    api_1.onMounted(function () { return runOnMount && start(); });
    api_1.onUnmounted(stop);
    return { observedEntry: observedEntry, start: start, stop: stop };
}
exports.useIntersection = useIntersection;
