"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var api_1 = require("@src/api");
var utils_1 = require("@src/shared/utils");
function useLocation(runOnMount) {
    if (runOnMount === void 0) { runOnMount = true; }
    var buildState = function (trigger) {
        var state = history.state, length = history.length;
        var hash = location.hash, host = location.host, hostname = location.hostname, href = location.href, origin = location.origin, pathname = location.pathname, port = location.port, protocol = location.protocol, search = location.search;
        return {
            trigger: trigger,
            state: state,
            length: length,
            hash: hash,
            host: host,
            hostname: hostname,
            href: href,
            origin: origin,
            pathname: pathname,
            port: port,
            protocol: protocol,
            search: search
        };
    };
    var isTracking = api_1.ref(false);
    var locationState = api_1.ref(buildState('load'));
    var popState = function () { return (locationState.value = buildState('popstate')); };
    var pushState = function () { return (locationState.value = buildState('pushstate')); };
    var replaceState = function () { return (locationState.value = buildState('replacestate')); };
    var start = function () {
        if (isTracking.value)
            return;
        utils_1.patchHistoryMethodsOnce();
        locationState.value = buildState('start');
        window.addEventListener('popstate', popState);
        window.addEventListener('pushstate', pushState);
        window.addEventListener('replacestate', replaceState);
        isTracking.value = true;
    };
    var stop = function () {
        if (!isTracking.value)
            return;
        window.removeEventListener('popstate', popState);
        window.removeEventListener('pushstate', pushState);
        window.removeEventListener('replacestate', replaceState);
        isTracking.value = false;
    };
    api_1.onMounted(function () { return runOnMount && start(); });
    api_1.onUnmounted(stop);
    return { locationState: locationState, isTracking: isTracking, start: start, stop: stop };
}
exports.useLocation = useLocation;
