"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var api_1 = require("@src/api");
var defaultState = {
    angle: 0,
    type: 'landscape-primary'
};
function useOrientation(initialState, runOnMount) {
    if (initialState === void 0) { initialState = defaultState; }
    if (runOnMount === void 0) { runOnMount = true; }
    var isTracking = api_1.ref(false);
    var orientation = api_1.ref(initialState);
    var handleOrientationChange = function () {
        if (screen.orientation) {
            var _a = screen.orientation, angle = _a.angle, type = _a.type;
            orientation.value = { angle: angle, type: type };
        }
        else if (window.orientation) {
            orientation.value = {
                angle: typeof window.orientation === 'number' ? window.orientation : 0,
                type: ''
            };
        }
        else {
            orientation.value = initialState;
        }
    };
    var start = function () {
        if (isTracking.value)
            return;
        window.addEventListener('orientationchange', handleOrientationChange);
        handleOrientationChange();
        isTracking.value = true;
    };
    var stop = function () {
        if (!isTracking.value)
            return;
        window.removeEventListener('orientationchange', handleOrientationChange);
        isTracking.value = false;
    };
    api_1.onMounted(function () { return runOnMount && start(); });
    api_1.onUnmounted(stop);
    return { orientation: orientation, isTracking: isTracking, start: start, stop: stop };
}
exports.useOrientation = useOrientation;
