"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var api_1 = require("@src/api");
var getFps = function (fps) { return Number(api_1.isRef(fps) ? fps.value : fps); };
var calcFpsInterval = function (fps) { return 1000 / fps; };
var fpsLimit = 60;
function useRafFn(callback, 
// Note: a value equal to or greater than 60 will disable the fps check logic
// giving maximum precision and smoothness
fps, runOnMount) {
    if (fps === void 0) { fps = fpsLimit; }
    if (runOnMount === void 0) { runOnMount = true; }
    var isRunning = api_1.ref(false);
    // Using a computed here allows us to update fps
    // dynamically from user's input
    var fpsInterval = api_1.computed(function () { return calcFpsInterval(getFps(fps)); });
    var itWasIdle = false;
    var startTime = 0;
    var timeElapsed = 0;
    var timePausedTot = 0;
    var timeDelta = 0;
    var loop = function (timeStamp) {
        if (!startTime)
            startTime = timeStamp;
        if (!isRunning.value)
            return;
        // Store the time that the loop is idle so that we can
        // calculate the timeElapsed variable correctly and return
        // it to the user from within the callback
        if (itWasIdle) {
            timePausedTot = timeStamp - startTime - timeElapsed;
            itWasIdle = false;
        }
        // Adjust timeElapsed to account for startTime and timePausedTot
        timeElapsed = timeStamp - startTime - timePausedTot;
        // Always run the callback if fps is greater than fpsLimit
        var callbackShouldAlwaysRun = getFps(fps) >= fpsLimit;
        if (callbackShouldAlwaysRun) {
            // Store timeDelta for future computations
            timeDelta = timeElapsed;
            callback(timeElapsed);
        }
        // Run callback only when !callbackShouldAlwaysRun
        // and the given fps matches the lapsed time
        if (!callbackShouldAlwaysRun) {
            var elapsedTimeFromPrevLoop = Math.ceil(timeElapsed - timeDelta);
            if (elapsedTimeFromPrevLoop > fpsInterval.value) {
                // Store timeDelta for future computations
                timeDelta = timeElapsed;
                callback(timeElapsed);
            }
        }
        // Run loop again recursively
        requestAnimationFrame(loop);
    };
    var start = function () {
        isRunning.value = true;
        requestAnimationFrame(loop);
    };
    var stop = function () {
        isRunning.value = false;
        itWasIdle = true;
    };
    api_1.onMounted(function () { return runOnMount && start(); });
    api_1.onUnmounted(stop);
    return {
        isRunning: isRunning,
        start: start,
        stop: stop
    };
}
exports.useRafFn = useRafFn;
