"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var api_1 = require("@src/api");
var utils_1 = require("@src/shared/utils");
var normalizeParams = function (urlParamsObj) { return function (paramAcc, param) {
    paramAcc[param] = param in urlParamsObj ? urlParamsObj[param] : null;
    return paramAcc;
}; };
var getUrlParams = function (parameters) {
    var urlParamsObj = utils_1.normalizeEntriesData(Array.from(new URLSearchParams(location.search).entries()));
    return parameters.reduce(normalizeParams(urlParamsObj), {});
};
function useSearchParams(parameters, runOnMount) {
    if (runOnMount === void 0) { runOnMount = true; }
    var searchParams = api_1.ref(parameters.reduce(normalizeParams({}), {}));
    var isTracking = api_1.ref(false);
    var handleParamsChange = function () {
        return (searchParams.value = getUrlParams(parameters));
    };
    var start = function () {
        if (isTracking.value)
            return;
        utils_1.patchHistoryMethodsOnce();
        handleParamsChange();
        window.addEventListener('popstate', handleParamsChange);
        window.addEventListener('pushstate', handleParamsChange);
        window.addEventListener('replacestate', handleParamsChange);
        isTracking.value = true;
    };
    var stop = function () {
        if (!isTracking.value)
            return;
        window.removeEventListener('popstate', handleParamsChange);
        window.removeEventListener('pushstate', handleParamsChange);
        window.removeEventListener('replacestate', handleParamsChange);
        isTracking.value = false;
    };
    api_1.onMounted(function () { return runOnMount && start(); });
    api_1.onUnmounted(stop);
    return { searchParams: searchParams, isTracking: isTracking, start: start, stop: stop };
}
exports.useSearchParams = useSearchParams;
