"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var api_1 = require("@src/api");
var errorMsg = 'ResizeObserver is not supported, please install a polyfill';
function useSize(elRef, options, runOnMount) {
    if (options === void 0) { options = {}; }
    if (runOnMount === void 0) { runOnMount = true; }
    var isTracking = api_1.ref(false);
    var observedEntry = api_1.ref(null);
    var handleObserver = function (entries) {
        observedEntry.value = entries[0];
    };
    var observer = null;
    var start = function () {
        if (isTracking.value)
            return;
        if (!('ResizeObserver' in window))
            throw new Error(errorMsg);
        // Do not start if the observer is already initialized
        // or the elRef does not exist
        if (!elRef.value)
            return;
        observer = new window.ResizeObserver(handleObserver);
        observer.observe(elRef.value, options);
        isTracking.value = true;
    };
    var stop = function () {
        if (!isTracking.value)
            return;
        if (!observer)
            return;
        observer.disconnect();
        isTracking.value = false;
    };
    api_1.onMounted(function () { return runOnMount && start(); });
    api_1.onUnmounted(stop);
    return { observedEntry: observedEntry, isTracking: isTracking, start: start, stop: stop };
}
exports.useSize = useSize;
