"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var api_1 = require("@src/api");
var utils_1 = require("@src/shared/utils");
function useWindowSize(runOnMount) {
    if (runOnMount === void 0) { runOnMount = true; }
    var width = api_1.ref(utils_1.isClient ? window.innerWidth : Infinity);
    var height = api_1.ref(utils_1.isClient ? window.innerHeight : Infinity);
    var isTracking = api_1.ref(false);
    var handleWindowResize = function () {
        width.value = window.innerWidth;
        height.value = window.innerHeight;
    };
    var start = function () {
        if (isTracking.value)
            return;
        window.addEventListener('resize', handleWindowResize);
        handleWindowResize();
        isTracking.value = true;
    };
    var stop = function () {
        if (!isTracking.value)
            return;
        window.removeEventListener('resize', handleWindowResize);
        isTracking.value = false;
    };
    api_1.onMounted(function () { return runOnMount && start(); });
    api_1.onUnmounted(stop);
    return { width: width, height: height, isTracking: isTracking, start: start, stop: stop };
}
exports.useWindowSize = useWindowSize;
