"use strict";
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
var utils_1 = require("@src/shared/utils");
var api_1 = require("@src/api");
var defaultOptions = {
    isParsing: false
};
function createStorage(storage, key, options) {
    var _a = Object.assign({}, defaultOptions, options), isParsing = _a.isParsing, opts = __rest(_a, ["isParsing"]);
    var serializer = utils_1.createSerializer(opts.serializer);
    var deserializer = utils_1.createDeserializer(opts.deserializer);
    var item = api_1.ref(null);
    var getItem = function () {
        try {
            var itemVal = utils_1.tryDeserialize(storage.getItem(key), deserializer, isParsing);
            if (!utils_1.isNullOrUndefined(itemVal))
                item.value = itemVal;
        }
        catch (error) {
            // If user is in private mode or has storage restriction
            // sessionStorage/locationStorage can throw
        }
    };
    var setItem = function (
    // The user may pass a 'string', a 'number', a valid JSON object/array
    // or even a custom object when serializer/deserializer are defined
    // so it is better to set allowed newItemVal as 'any'
    newVal) {
        try {
            var newItemVal = utils_1.trySerialize(newVal, serializer, isParsing);
            storage.setItem(key, newItemVal);
            item.value = utils_1.tryDeserialize(newItemVal, deserializer, isParsing);
        }
        catch (error) {
            // If user is in private mode or has storage restriction
            // sessionStorage/locationStorage can throw
        }
    };
    var removeItem = function () {
        try {
            storage.removeItem(key);
            item.value = null;
        }
        catch (error) {
            // If user is in private mode or has storage restriction
            // sessionStorage/locationStorage can throw
            item.value = null;
        }
    };
    return {
        item: item,
        getItem: getItem,
        setItem: setItem,
        removeItem: removeItem
    };
}
exports.createStorage = createStorage;
