"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.isClient = typeof window === 'object';
var checkType = function (typeToCheck) {
    return Object.prototype.toString.call(typeToCheck);
};
exports.isString = function (varToCheck) {
    return checkType(varToCheck) === '[object String]';
};
exports.isObj = function (varToCheck) {
    return checkType(varToCheck) === '[object Object]';
};
exports.isNull = function (varToCheck) {
    return checkType(varToCheck) === '[object Null]';
};
exports.isFunction = function (varToCheck) {
    return checkType(varToCheck) === '[object Function]';
};
exports.isUndefined = function (varToCheck) {
    return checkType(varToCheck) === '[object Undefined]';
};
exports.isNullOrUndefined = function (varToCheck) {
    return exports.isNull(varToCheck) || exports.isUndefined(varToCheck);
};
// The history methods 'pushState' and 'replaceState' by default do not fire an event
// unless it is coming from user interaction with the browser navigation bar,
// so we are adding a patch to make them detectable
var isPatched = false;
exports.patchHistoryMethodsOnce = function () {
    if (isPatched)
        return;
    var methods = ['pushState', 'replaceState'];
    methods.forEach(function (method) {
        var original = history[method];
        history[method] = function (state) {
            // eslint-disable-next-line prefer-rest-params
            var result = original.apply(this, arguments);
            var event = new Event(method.toLowerCase());
            event.state = state;
            window.dispatchEvent(event);
            return result;
        };
    });
    isPatched = true;
};
exports.normalizeEntriesData = function (data) {
    return data.reduce(function (acc, _a) {
        var key = _a[0], val = _a[1];
        acc[key] = val;
        return acc;
    }, {});
};
exports.createSerializer = function (serializer) {
    return serializer || JSON.stringify;
};
exports.createDeserializer = function (deserializer) {
    return deserializer || JSON.parse;
};
var fallbackToString = function (val) { return (exports.isString(val) ? val : String(val)); };
exports.trySerialize = function (val, serializer, shouldSerialize) {
    if (!shouldSerialize)
        return fallbackToString(val);
    try {
        return fallbackToString(serializer(val));
    }
    catch (error) {
        return fallbackToString(val);
    }
};
exports.tryDeserialize = function (val, deserializer, shouldDeserialize) {
    if (!shouldDeserialize)
        return val;
    try {
        return deserializer(val);
    }
    catch (error) {
        return val;
    }
};
